#!/usr/bin/env python

#   reStructWeb v0.2 - A very light reStructured text Web system
#   Version : 2004/05/26
#   David ROBERT david@ombrepixel.com
#   http://www.ombrepixel.com/drobert/restructweb.html

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# ---------  You may need to configure theses parameters ---------

# This is the place where everyting lives :
# generated pages, reST pages, templates
folder='/home/web/david/pages'

# Magicword (password for update)
magicWord='gigi'

# Language you want to be available (edit and see)
supportedLangs=['fr','en'] 

# Default Language to use (only in standalone mode)
defaultLang='fr'

# Encoding (of the generated pages)
encoding='utf-8'

# -----------------  End of configuration  ------------------------

import cgi
import cgitb; cgitb.enable()
import sys, os, time
import string, re
import docutils.core
from docutils import nodes, utils
from docutils.readers import standalone
from docutils.transforms import TransformError, Transform

class page:
	def __init__(self,pageName,lang):
		self.lang = lang
		self.pageName = pageName
		self.fileName = "%s/%s" % (folder,self.pageName)
		self.htmlName = "%s.html.%s" % (self.fileName,self.lang)
		self.rstName = "%s.txt.%s" % (self.fileName,self.lang)

		
	def getContent(self):
		"Return the Web page content"
		if self.htmlExist():
			if not self.isHtmlUptodate(): self.updateHtml()
		else:
			if self.rstExist():
				self.updateHtml()
			else:
				return self.newRst()
		return self.getHtml()

	def htmlExist(self):
		return os.path.isfile(self.htmlName)
		
	def rstExist(self):
		return os.path.isfile(self.rstName)
		
	def getHtml(self):
		"Return html file content"
		try:
			f=open(self.htmlName, "r")
			return f.read()
		except:
			return "Error opening file: %s" % self.htmlName
			
	def getRst(self,myLang=None):
		"Return rst page content"
		if myLang==None: myLang=self.lang
		try:
			f=open(self.fileName + ".txt." + myLang, "r")
			return f.read()
		except:
			return "This page is empty, try another language"
			
	def saveRst(self,text):
		"Save rst files from edit page"
		for (myLang, myText) in text.items():
			filename = self.fileName + ".txt." + myLang
			try:
				os.rename(filename,filename + ".old")
			except:
				pass # Maybe the html file don't exist
			try:
				f=open(filename, "w")
				f.write(myText)
				f.close()
			except:
				print "Could not save %s rst file" % myLang	

	def isHtmlUptodate(self):
		"Return true if html is older that rst source file"
		# get st_mtime (time of most recent content modification)
		epochRst=os.stat(self.rstName)[8]
		epochHtml=os.stat(self.htmlName)[8]
		if epochHtml > epochRst: return 1
		else: return 0
		
	def updateHtml(self):
		"Generate html content from rst txt source file"

		docDict=docutils.core.publish_parts( source=self.getRst(),
		reader=Reader(),
		writer_name='html')
		
		body=docDict["body"].encode(encoding)
		title=docDict["title"].encode(encoding)
		try:
			html=self.templateIt(title,body)
		except:
			html=docDict["whole"].encode(encoding)
		
		f=open(self.htmlName, "w")
		f.write(html)
		f.close()

	def templateIt(self,title,body):
		"Need, title and body, return html using template"

		# Page dedicated template or default template	
		if os.path.isfile("%s/%s.tmpl.%s" % (folder, self.pageName, self.lang)):
			self.template = "%s/%s.tmpl.%s" % (folder, self.pageName, self.lang)	
		else:
			self.template = "%s/default.tmpl.%s" % (folder, self.lang)
		h=open(self.template,"r")
		html = h.read()
		h.close()

		# Template integration job
		html = html.replace("$sourceUrl","%s.txt.%s" % ( self.pageName, self.lang))
		html = html.replace("$editUrl","%s.html.%s?action=edit" % (self.pageName,self.lang))
		html = html.replace("$frUrl","%s.html.fr" % (self.pageName))
		html = html.replace("$enUrl","%s.html.en" % (self.pageName))
		html = html.replace("$lastModified","%s" % (time.strftime("%Y-%m-%d %T %Z")))
		html = html.replace("$title",title)
		html = html.replace("$body",body)

		return html
			
	
	def editPage(self):
		"Return editing page"
		title="Editing page: %s" % self.pageName
		body="""<table width="100%%">
		<tr>
		<td align="left"> Editing page : <b>%s</b> </td>
		<td align="right">
 		<a href="http://docutils.sourceforge.net/docs/rst/quickref.html">
		Help about reStructuredText</a> </td>
		</tr></table>
		<hr />
		<form action="%s.html.%s" method="POST">
		You need the MagicWord :-) 
		<input type="password" name="magicword">
		<input name="submit" value="Apply" type="submit"><br />
		If this page should be a link, type its name : 
		<input type="text" name="link" value="">
		""" % (self.pageName, self.pageName, self.lang)

		for thisLang in supportedLangs:
			try:
                        	text = self.getRst(thisLang)
                	except:
                        	text= ""

			body += """ Describe in %s<br>
			<textarea name="rstcontent%s" rows="25" cols="80">%s</textarea><p>
			""" % (thisLang, thisLang, text)
		
		body += """ <input type="hidden" name="action" value="save">
		<input type="hidden" name="page" value="%s">
		</form>
		</body>
		</html>""" % self.pageName
		try:
			html = self.templateIt(title,body)
		except:
			html = body
		return html


	def newRst(self):
		"""Return a web page with a link to create a new page
		in case of Apache 404 redirect, the html must be greater than
		512 bytes to be visible in Ms Internet Explorer"""

		title="This page don't exist, create one ?"
		body=""" <!-- We want IE to show this page (512 bytes): %s --> 
			This page has to be <a href="%s.html.%s?action=edit">created</a>
			<br>Come back to the <a href="index.html.%s">index page</a>
			""" % ("a"*512, self.pageName, self.lang, self.lang)
		try:
			html = self.templateIt(title,body)
		except:
			html = body
		return html
	
	def getSourceContent(self):
		return "<html><head></head><body><pre>%s</pre></body></html>" % self.getRst()
		
	def createLinks(self,link):
		import posix
		for myLang in supportedLangs:
			filename = self.fileName + ".txt." + myLang 
			try:
				posix.symlink(folder + "/" + link + ".txt." + myLang ,filename)
			except:
				print "Link failure for" + filename
		

def clean(url):
	return re.compile(r'[^a-z0-9]').sub('', url.lower())

# Define a new transformer
class AutoLinkResolver(nodes.SparseNodeVisitor):
	"Create autolink on unresolved nodes"
	def visit_reference(self, node):
        	if not node.resolved and node.hasattr('refname'):
        		refname = node['refname']
		        node.resolved = 1
		        node['class'] = 'autolink'
		        refuri = clean(refname)
			node['refuri'] = refuri + ".html." + lang 
		        del node['refname']

class AutoLink(Transform):
	default_priority = 830

	def apply(self):
		visitor = AutoLinkResolver(self.document)
        	self.document.walk(visitor)
        
class Reader(standalone.Reader):
	supported = standalone.Reader.supported + ('autolink',)
	default_transforms = standalone.Reader.default_transforms  + (AutoLink,)


#########################################
## No pain but main
##
#########################################

if __name__ == '__main__':
	form = cgi.FieldStorage()

	# When working with apache Redirect
	# ie: mod_action redirect or 404 redirect
	if os.environ.has_key('REDIRECT_STATUS'):
		url = os.environ['REDIRECT_URL']
		pattern=r'^.*\/(.+)\.html\.(..)$'
		r = re.compile(pattern)
		m = r.match(url)
		if m == None:
			print "Location: index.html.en\n" 
			sys.exit(0)
			
		pageName = m.group(1)
		lang = m.group(2)
	
	# Working in standalone mode
	else:
		# What page do we have to serve (in PATH_INFO)
		try:
			pathinfo=os.environ['PATH_INFO']
			pattern=r'^\/(.+)\.html\.(..)$'
			r = re.compile(pattern)
			m = r.match(pathinfo)
			if m == None:
				print "Location: index.html.en\n" 
				sys.exit(0)
			
			pageName = clean(m.group(1))
			lang = m.group(2)
			
			if pageName == "" or len(pageName) > 30:
				raise ValueError
		except:
			print "Location: index.html.en\n" 
			sys.exit(0)


		# In standalone mode, detect webbrowser language
		# preference :

		## Maybe lang is provided in the request
		#if form.has_key("lang") and form.getvalue("lang") in supportedLangs:
		#	lang=form.getvalue("lang")
	        #
		# else we look languages accepted by browser
		#else:
		#	# Which language the browser want
		#	try:
		#		language=os.environ['HTTP_ACCEPT_LANGUAGE']
		#		pattern=r'^(..).*$'
		#		r = re.compile(pattern)
		#		m = r.match(language)
		#		l = m.group(1)
		#		if l in supportedLangs:
		#			lang=l
		#		else:
		#			lang=defaultLang
		#	except:
		#		lang=defaultLang

	myPage = page(pageName,lang)

	print "Content-Type: text/html; charset=%s\n" % encoding

	action = form.getvalue("action")

	if action == "edit":
		print myPage.editPage()
		
	elif action == 'save':
		password=form.getvalue("magicword")
		if password == magicWord: # That's good
			link=form.getvalue("link")

			# Just create links
			if link and len(link) > 0:
				myPage.createLinks(clean(link))
				print myPage.getContent()
			# Else create real page
			else:
				text = {} 
				for myLang in supportedLangs:
					text[myLang]=form.getvalue("rstcontent"+myLang)
				myPage.saveRst(text)
				print myPage.getContent()
		
	elif action == 'source':
		print myPage.getSourceContent()

	else:
		print myPage.getContent()

	sys.exit(0)
